@@ -78,6 +78,7 @@ make_qrcode <- function(my_id, ec_level = 3){
 #' @param bottom_left_1 String for bottom-left row 1 position on a rectangular label.
 #' @param bottom_left_2 String for bottom-left row 2 position on a rectangular label.
 #' @param unique_id A vector containing unique identifiers or strings to generate QR codes.
+#' @param include_qr Logical. Set to FALSE to exclude QR codes. Default is TRUE.
 #' @param ec_level The error correction level (`0` - `3`, lowest to highest) for QR codes.
 #' @param ... Additional optional arguments to be supplied.
 #' 
@@ -140,6 +141,7 @@ create_label <- function(
     bottom_left_1 = NULL, # Text for bottom-left row 1
     bottom_left_2 = NULL, # Text for bottom-left row 2
     unique_id = NULL, # Unique ids for QR codes
+    include_qr = TRUE,
     ec_level = 3,
     ...){
   
@@ -199,34 +201,6 @@ create_label <- function(
   
   assertthat::assert_that(assertthat::is.string(family),
                           msg = paste("'family'", error_string))
-  
-  # assertthat::assert_that(assertthat::is.string(top_left_1),
-  #                         msg = paste("'top_left_1'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(top_left_2),
-  #                         msg = paste("'top_left_2'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(top_right_1),
-  #                         msg = paste("'top_right_1'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(top_right_2),
-  #                         msg = paste("'top_right_2'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(center_right_1),
-  #                         msg = paste("'center_right_1'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(center_right_2),
-  #                         msg = paste("'center_right_2'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(center_right_3),
-  #                         msg = paste("'center_right_3'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(bottom_left_1),
-  #                         msg = paste("'bottom_left_1'", error_string))
-  #
-  # assertthat::assert_that(assertthat::is.string(bottom_left_2),
-  #                         msg = paste("'bottom_left_2'", error_string))
-  
   ## -- Assertions end
   
   # Calculate space between label columns if any
@@ -320,16 +294,31 @@ create_label <- function(
   }
   
   # Create QR codes from unique ids
-  if (!is.null(unique_id)) {
+  
+  if (include_qr) {
+    
+    if (is.null(unique_id)) {
+      stop("`unique_id` is required when `include_qr = TRUE`.")
+    }
+    
     bb <- unique_id |> purrr::map(\(x) make_qrcode(ec_level = ec_level, x))
     nn <- length(bb) # total number of labels to generate
+    
   } else {
-    stop("Unique IDs for generating QR codes are missing!!")
+    
+    text_fields <- list(
+      top_left_1, top_left_2,
+      top_right_1, top_right_2,
+      center_right_1, center_right_2, center_right_3,
+      bottom_left_1, bottom_left_2)
+    
+    text_lengths <- vapply(text_fields, function(x) if (is.null(x)) 0 else length(x), numeric(1))
+    nn <- max(text_lengths, 1)
+    bb <- NULL
   }
   
-  
   # Generate label positions -- prints across rows of grid layout
-  if (print_across == TRUE) {
+  if (print_across) {
     
     pos <- expand.grid(x = 1:numcol, y = 1:numrow)
     
@@ -381,7 +370,7 @@ create_label <- function(
                                     widths = grid::unit(c(rep(wdt + col_space, numcol-1), wdt), "in"),
                                     heights = grid::unit(c(rep(hgt + row_space, numrow-1), hgt), "in"))
   
-  if (Treetag == TRUE) {
+  if (Treetag) {
     # Viewport for first QR codes
     qq <- grid::viewport(x = grid::unit(0.57, "npc"),
                          y = grid::unit(0.3, "npc"),
@@ -474,6 +463,7 @@ create_label <- function(
   lab_vp <- grid::viewport(layout = label_layout)
   grid::pushViewport(lab_vp)
   
+  # Fonts for text display
   bold_font <- grid::gpar(fontface = "bold", fontsize = fsize)
   bold_font2 <- grid::gpar(fontface = "bold", fontsize = fsize-2)
   plain_font2 <- grid::gpar(fontface = "plain", fontsize = fsize-2)
@@ -500,16 +490,16 @@ create_label <- function(
       
     }
     
-    if (Treetag == TRUE) {
+    if (Treetag) {
       tt <- grid::viewport(layout.pos.row=label_posn['y'],
                            layout.pos.col=label_posn['x'])
       
       grid::pushViewport(tt)
       
       # Draw rectangle around labels
-      if (rect == TRUE) {
+      if (rect) {
         
-        if (rounded == TRUE) {
+        if (rounded) {
           grid::grid.roundrect(gp = grid::gpar(lwd = 0.5))
         } else (grid::grid.rect(gp = grid::gpar(lwd = 0.5)))
         
@@ -595,6 +585,17 @@ create_label <- function(
                       y = grid::unit(ey, "in"),
                       gp = plain_font4, hjust = 0, vp = tt)
       
+      # Add 'qrlabelr' branding at the extreme right edge
+      grid::grid.text(
+        label = "qrlabelr",
+        x = grid::unit(0.5 * wdt, "in"),
+        y = grid::unit(0.02 * hgt, "in"),
+        rot = 180,
+        gp = grid::gpar(fontface = "plain", fontsize = plain_font4, 
+                        col = "gray30"),
+        just = "center",
+        vp = tt)
+      
       grid::pushViewport(qq2)
       
       # Add second QR code
@@ -609,9 +610,9 @@ create_label <- function(
       grid::pushViewport(aa)
       
       # Draw rectangle around labels
-      if (rect == TRUE) {
+      if (rect) {
         
-        if (rounded == TRUE) {
+        if (rounded) {
           grid::grid.roundrect(gp = grid::gpar(lwd = 0.5))
         } else (grid::grid.rect(gp = grid::gpar(lwd = 0.5)))
         
@@ -654,7 +655,6 @@ create_label <- function(
                       gp = plain_font3, hjust = 0, vp = aa)
       
       # Push viewport for qr code
-      
       grid::pushViewport(qq)
       
       # Add QR code
@@ -662,7 +662,6 @@ create_label <- function(
       grid::popViewport()
       
       # Define viewport for Block ID if it is an incomplete block design
-      
       grid::pushViewport(bl)
       
       # Add Block ID to label; note coordinates
@@ -688,6 +687,17 @@ create_label <- function(
                       y = grid::unit(ey, "in"),
                       gp = bold_font3, hjust = 0, vp = aa)
       
+      # Add 'qrlabelr' branding at the extreme left edge
+      grid::grid.text(
+        label = "qrlabelr",
+        x = grid::unit(0.02 * wdt, "in"),
+        y = grid::unit(0.5 * hgt, "in"),
+        rot = 90,
+        gp = grid::gpar(fontface = "plain", fontsize = plain_font4, 
+                        col = "gray30"),
+        just = "center",
+        vp = aa)
+      
       grid::popViewport(2)
     }
     
@@ -860,7 +870,7 @@ field_label <- function(dat,
     
   }
   
-  if (seed_source == TRUE & is.null(seed_source_id)) {
+  if (seed_source & is.null(seed_source_id)) {
     stop(paste("Use the 'seed_source_id' argument to specify the column name in your fieldbook containing seed source ids."))
   }
   
@@ -906,7 +916,7 @@ field_label <- function(dat,
   # }
   
   # Define year for experiment
-  if (is.null(Year) == TRUE) {
+  if (is.null(Year)) {
     yr <- format(Sys.time(),'%Y')
   } else {
     yr <- as.character(Year)
@@ -927,7 +937,7 @@ field_label <- function(dat,
   rnames <- rep(rname, nrow(dat)) # Researcher's name
   
   # Block ids if it is an incomplete block design
-  if (IBlock == TRUE) {
+  if (IBlock) {
     iblock <- dat[, IBlock_id]
     blkid <- paste("B:", iblock)
   } else {
@@ -936,7 +946,7 @@ field_label <- function(dat,
   
   
   # Show seed source on label
-  if (seed_source == TRUE) {
+  if (seed_source) {
     
     sds <- paste(dat[, seed_source_id])
     
@@ -950,6 +960,8 @@ field_label <- function(dat,
   # If 'custom', user must use the 'unique_id' argument to specify the column name
   # in field book that denotes custom generated unique IDs.
   
+  get_unique_id <- match.arg(get_unique_id)
+  
   if (get_unique_id == "ruid"){
     
     trial_name <- gsub(" ", "_", Trial)
@@ -980,8 +992,7 @@ field_label <- function(dat,
   }
   
   # File name and timestamp for updated fieldbook
-  ts <- Sys.time() |> format(format = "%H_%M_%S") |> 
-    paste0("Updated_Fieldbook_", ... = _, ".csv")
+  ts <- paste0("Updated_Fieldbook_", format(Sys.time(), format = "%H_%M_%S"), ".csv")
   
   utils::write.csv(updat, file = paste0(filename, ts), row.names = FALSE)
   
@@ -1213,6 +1224,8 @@ gp_label <- function(dat,
   # If 'custom', user must use the 'unique_id' argument to specify the column name
   # in fieldbook that denotes custom generated unique IDs.
   
+  get_unique_id <- match.arg(get_unique_id)
+  
   if (get_unique_id == "uuid") {
     UNIQUE_ID <- uuid::UUIDgenerate(use.time = TRUE, n = nrow(dat), output = "string")
     
@@ -1231,8 +1244,7 @@ gp_label <- function(dat,
   }
   
   # File name and timestamp for updated fieldbook
-  ts <- Sys.time() |> format(format = "%H_%M_%S") |> 
-    paste0("Updated_Fieldbook_", ... = _, ".csv")
+  ts <- paste0("Updated_Fieldbook_", format(Sys.time(), format = "%H_%M_%S"), ".csv")
   
   utils::write.csv(updat, file = paste0(filename, ts), row.names = FALSE)
   
@@ -1315,6 +1327,7 @@ gp_label <- function(dat,
 #' @param top_txt2_id The column identifier in \code{dat} containing text for top text position 2.
 #' @param top_txt3_id The column identifier in \code{dat} containing text for top text position 3.
 #' @param unique_id The column identifier in \code{dat} containing unique identifiers or strings to generate QR codes.
+#' @param include_qr Logical. Set to FALSE to exclude QR codes. Default is TRUE.
 #' @param ec_level The error correction level (`0` - `3`, lowest to highest) for QR codes.
 #' 
 #' @seealso \code{\link{field_label}} and \code{\link{gp_label}}
@@ -1415,8 +1428,8 @@ gp_label_portrait <- function(
     top_txt2_id = NULL, # Top position text 2 column ID
     top_txt3_id = NULL, # Top position text 3 column ID
     unique_id = NULL, # Column ID for Unique ids for QR codes
+    include_qr = TRUE,
     ec_level = 3 # qr code error correction level
-    
 ) {
   
   error_numeric <- "must be a positive numeric value."
@@ -1600,20 +1613,34 @@ gp_label_portrait <- function(
   }
   
   # Create QR codes from unique ids
-  if (!is.null(unique_id)) {
+  if (include_qr) {
+    
+    if (is.null(unique_id)) {
+      stop("`unique_id` is required when `include_qr = TRUE`.")
+    }
+    
     bb <- dat[, unique_id] |> purrr::map(\(x) make_qrcode(ec_level = ec_level, x))
     nn <- length(bb) # total number of labels to generate
-  } else if (is.null(unique_id)) {
-    stop("Unique IDs for generating QR codes are missing!!")
+    
+  } else {
+    
+    text_fields <- list(
+      text1, text2,
+      text3, text4,
+      text5, text6, 
+      text7, text8, 
+      text9, text10)
+    
+    text_lengths <- vapply(text_fields, function(x) if (is.null(x)) 0 else length(x), numeric(1))
+    nn <- max(text_lengths, 1)
+    bb <- NULL
   }
   
-  
   # clean up any open graphical devices if function fails
   on.exit(grDevices::graphics.off())
   
-  
   # Generate label positions -- prints across rows of grid layout
-  if (print_across == TRUE) {
+  if (print_across) {
     
     pos <- expand.grid(x = 1:numcol, y = 1:numrow)
     
@@ -1696,8 +1723,6 @@ gp_label_portrait <- function(
   
   txt10x <- 0.05*wdt # y coordinate for txt10
   
-  
-  
   # Viewport for a new page
   new.page <- grid::viewport(width = grid::unit(page_wdt, "in"),
                              height = grid::unit(page_hgt, "in"))
@@ -1739,9 +1764,9 @@ gp_label_portrait <- function(
     grid::pushViewport(aa)
     
     # Draw rectangle around labels
-    if (rect == TRUE) {
+    if (rect) {
       
-      if (rounded == TRUE) {
+      if (rounded) {
         grid::grid.roundrect(gp = grid::gpar(lwd = 0.5))
       } else (grid::grid.rect(gp = grid::gpar(lwd = 0.5)))
       
@@ -1778,16 +1803,13 @@ gp_label_portrait <- function(
                     y = grid::unit(txt1y, "in"), rot = 90,
                     gp = bold_font2, just = c(-0.5, 0.5), vp = aa)
     
-    
     # Define viewport for QR code; note coordinates and dimension
-    
     grid::pushViewport(qq)
     
     # Add first QR code
     grid::grid.draw(bb[[i]])
     # grid::popViewport()
     
-    
     # Go back to label viewport
     grid::upViewport()
     
@@ -1797,45 +1819,47 @@ gp_label_portrait <- function(
                     y = grid::unit(txt1y, "in"), rot = 90,
                     gp = bold_font3, hjust = 0, vp = aa)
     
-    
     # Add text7 to label (center position 4); note coordinates
     grid::grid.text(label = text7[i],
                     x = grid::unit(txt6x, "in"),
                     y = grid::unit(txt7y, "in"), rot = 90,
                     gp = bold_font3, hjust = 0, vp = aa)
     
-    
     # Add text8 to label (top position 1); note coordinates
     grid::grid.text(label = text8[i],
                     x = grid::unit(txt8x, "in"),
                     y = grid::unit(txt1y, "in"), rot = 90,
                     gp = bold_font3, hjust = 0, vp = aa)
     
-    
     # Add text9 to label (top position 2); note coordinates
     grid::grid.text(label = text9[i],
                     x = grid::unit(txt8x, "in"),
                     y = grid::unit(txt7y, "in"), rot = 90,
                     gp = bold_font3, hjust = 0, vp = aa)
     
-    
     # Add text10 to label (top position 3); note coordinates
     grid::grid.text(label = text10[i],
                     x = grid::unit(txt10x, "in"),
                     y = grid::unit(txt1y, "in"), rot = 90,
                     gp = bold_font, hjust = 0, vp = aa)
     
+    # Add 'qrlabelr' branding at the extreme left edge
+    grid::grid.text(
+      label = "qrlabelr",
+      x = grid::unit(0.7 * wdt, "in"),
+      y = grid::unit(0.96 * hgt, "in"),
+      gp = grid::gpar(fontface = "plain", fontsize = bold_font4, 
+                      col = "gray30"),
+      just = "center",
+      vp = aa)
     
     grid::popViewport(2)
     
-    
   } # End of create_label ()
   
-  #cat("\n\n\tGenerated labels saved to working directory as a PDF file.")
 } 
 
 
-
 #' Make an enhanced field layout plot with border rows. 
 #' 
 #' @description
